from time import strptime, strftime, mktime, gmtime
from datetime import datetime
from collections import OrderedDict
import logging

# since duply has a line for line output we can do this per rule


def calcDuration(startTime, endTime):
    strucTime = strptime(startTime, "%Y-%m-%d %H:%M:%S")
    t1 = datetime.fromtimestamp(mktime(strucTime))
    strucTime = strptime(endTime, "%Y-%m-%d %H:%M:%S")
    t2 = datetime.fromtimestamp(mktime(strucTime))
    duration = ((t2) - (t1))
    return str(duration)


def timeConvert(s):
    t = gmtime(float(s))
    return strftime("%Y-%m-%d %H:%M:%S", t)


def sizeOutOfOutput(string):
    # Split duply Size-String and return Size in KB
    size = int(string.split(" ")[0])
    sizeKB = size / 1024
    return str(sizeKB)


class duply():
    def __init__(self):
        '''constructor'''

    columns = OrderedDict([
       ('startTime', ['StartTime', 'DATETIME']),
       ('endTime', ['EndTime', 'DATETIME']),
       ('duration', ['ElapsedTime', 'TIME']),
       ('backupSize', ['ChangedFileSize', 'varchar(512)']),
       ('dp_errors', ['Errors', 'varchar(512)']),
       ('dp_sourceFiles', ['SourceFiles', 'varchar(512)']),
       ('dp_sourceSize', ['SourceFileSize', 'varchar(512)']),
       ('dp_newFiles', ['NewFiles', 'varchar(512)']),
       ('dp_newFileSize', ['NewFileSize', 'varchar(512)']),
       ('dp_deletedFiles', ['DeletedFiles', 'varchar(512)']),
       ('dp_changedFiles', ['ChangedFiles', 'varchar(512)']),
       ('dp_changedDeltaSize', ['ChangedDeltaSize', 'varchar(512)']),
       ('dp_deltaEntries', ['DeltaEntries', 'varchar(512)']),
       ('dp_rawSize', ['RawDeltaSize', 'varchar(512)']),
       ('dp_destSize', ['TotalDestinationSizeChange', 'varchar(512)']),
       ('dp_errors', ['Errors', 'varchar(512)'])
    ])


    def parse(self, backupOutput):
        entries = OrderedDict([])
        output = ''
        try:
            for line in backupOutput:
                output += line
                if '/pre'in line:
                    try:
                        entries['precommand'] = line.split('- ')[1].rstrip('\n')
                    except:
                        logging.error("Problem Parsing pre line")
                for col in self.columns:
                    if self.columns[col][0] in line:
                        try:
                            value = line.rstrip('\n')
                            value = value.strip(self.columns[col][0] + " ")
                            if 'backupSize' in col:
                                # we want have the real size in kb
                                value = sizeOutOfOutput(value)
                            if 'Time' in col:
                                value = timeConvert(value.split(' ')[0])
                            entries[col] = value
                        except:
                            logging.error("Problem Parsing the line with" + col)
            entries['duration'] = calcDuration(entries['startTime'],
                                               entries['endTime'])
            entries['output'] = output
            entries['backupSuccess'] = 1
            return entries
        except KeyError:
            # on key error we assume the output was not as expected and the
            # backup went wrong
            logging.info("Could not parse the Output. Key error")
            entries['output'] = output
            entries['backupSuccess'] = 0
            return entries
        except:
            logging.info("Could not parse the Output. Unknown reason")
            entries['output'] = output
            entries['backupSuccess'] = 0
            return entries

    def getColumns(self):
        return self.columns

    def addColumns(self):
        print('not yet implemented')
