# 2017, Jan Kowalsky <jankow@datenkollektiv.net>
#
# This file is part of pybackcheck.
#
# pybackcheck is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pybackcheck is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pybackcheck.  If not, see <http://www.gnu.org/licenses/>.
#
# The main part of the forward tunnel code is distributed as a example of
# paramiko. Copyright (C) 2003-2007  Robey Pointer <robeypointer@gmail.com>
#

"""
Sample script showing how to do local port forwarding over paramiko.

This script connects to the requested SSH server and sets up local port
forwarding (the openssh -L option) from a local port through a tunneled
connection to a destination reachable from the SSH server machine.
"""

import os
import logging
from threading import Thread
import select
try:
    import SocketServer
except ImportError:
    import socketserver as SocketServer


import paramiko


localServer = 'localhost'


class ForwardServer (SocketServer.ThreadingTCPServer):
    daemon_threads = True
    allow_reuse_address = True


class Handler (SocketServer.BaseRequestHandler):

    def handle(self):
        try:
            chan = self.ssh_transport.open_channel('direct-tcpip',
                                                   (self.chain_host,
                                                    self.chain_port),
                                                   self.request.getpeername())
        except Exception as e:
            logging.warning('Incoming request to %s:%d failed: %s' % (self.chain_host,
                                                              self.chain_port,
                                                              repr(e)))
            return
        if chan is None:
            logging.warning('Incoming request to %s:%d was rejected by the SSH server.' %
                    (self.chain_host, self.chain_port))
            return

        logging.debug('Connected!  Tunnel open %r -> %r -> %r' %
                      (self.request.getpeername(),
                       chan.getpeername(), (self.chain_host, self.chain_port)))
        while True:
            r, w, x = select.select([self.request, chan], [], [])
            if self.request in r:
                data = self.request.recv(1024)
                if len(data) == 0:
                    break
                chan.send(data)
            if chan in r:
                data = chan.recv(1024)
                if len(data) == 0:
                    break
                self.request.send(data)

        peername = self.request.getpeername()
        chan.close()
        self.request.close()
        logging.debug('Tunnel closed from %r' % (peername,))


def forward_tunnel(transport):
    # this is a little convoluted, but lets me configure things for the Handler
    # object.  (SocketServer doesn't give Handlers any way to access the outer
    # server normally.)
    class SubHander (Handler):
        chain_host = localServer
        chain_port = forwardPort
        ssh_transport = transport
    global theserver
    theserver = ForwardServer(('', localPort), SubHander)
    theserver.serve_forever()
    logging.info('Tunnel opened from %r' % (theserver,))


def sshTunnel():
    logging.info('Fount private Key in %r' %(privateKey,))
    logging.info('Fount known_hosts in %r' %(hostKey,))
    client = paramiko.SSHClient()
    client.load_system_host_keys()
    if os.path.isfile(hostKey) is True:
        client.load_host_keys(hostKey)
    client.set_missing_host_key_policy(paramiko.WarningPolicy())
    if os.path.isfile(privateKey) is True:
        client.connect(remoteServer, port=remotePort, username=userName, key_filename=privateKey)
    else:
        client.connect(remoteServer, port=remotePort, username=userName)
    forward_tunnel(client.get_transport())

class tunnelThread(Thread):
    def __init__(self):
        '''Constructor'''
        Thread.__init__(self)
        # self.remoteServer = remoteServer
        # self.remotePort = remotePort
        # self.localPort = localPort
        # self.forwardPort = forwardPort
        # self.userName = userName

    def run(self):
        sshTunnel()


def start(remote_server, remote_port, local_port, forward_port, user_name,
          host_key, private_key):
    global remoteServer
    global remotePort
    global localPort
    global forwardPort
    global userName
    global hostKey
    global privateKey
    remoteServer = remote_server
    remotePort = remote_port
    localPort = local_port
    forwardPort = forward_port
    userName = user_name
    hostKey = host_key
    privateKey = private_key

    global e
    # e = futures.ThreadPoolExecutor(max_workers=1)
    try:
        e = tunnelThread()
        e.start()
    except:
        print('Cant start Tunnel')


def stop():
    try:
        theserver.shutdown()
        logging.info('Tunnel closed from %r' % (theserver,))
        e.join()
    except:
        print('Can\'t stop Tunnel')
